import { ColorMode, ControllerInfo, FontSize, IRectangle, ObjectType, RenderMode, ShuttleMode, TBarResolution, TextRenderOptions } from './protocol';
import { TypedEmitter } from 'tiny-typed-emitter';
import { TCPClientPortSettings } from './tcp-client';
import { TCPServerPortSettings } from './tcp-server';
import * as Buttons from './buttons';
export { ObjectType, FontSize, ShuttleMode, ColorMode, ControllerInfo, IRectangle, RenderMode, TBarResolution, Buttons };
export declare const FullScreen: IRectangle;
export interface JLCSlomoEliteEvents {
    'button': (button: Buttons.ButtonType, down: boolean) => void;
    'jog': (jog: number) => void;
    'shuttle': (pos: number) => void;
    'tbar': (pos: number) => void;
    'connected': () => void;
    'disconnected': () => void;
}
export interface SerialPortSettings {
    type: 'serial';
    port: string;
}
export declare class JLCSlomoElite extends TypedEmitter<JLCSlomoEliteEvents> {
    private port;
    private protocol;
    private connected_;
    constructor(port: SerialPortSettings | TCPClientPortSettings | TCPServerPortSettings);
    get connected(): boolean;
    private setupSerialPort;
    private setupTCPClient;
    private setupTCPServer;
    private openPort;
    dispose(): void;
    open(): Promise<void>;
    clear(): Promise<void>;
    private send;
    requestSystemInfo(): Promise<ControllerInfo>;
    drawText(data: string, options?: TextRenderOptions): Promise<void>;
    setShuttleMode(mode: ShuttleMode): Promise<void>;
    setButtonColor(button: Buttons.ButtonType, color: ColorMode): Promise<void>;
    setTBarResolution(resolution: TBarResolution): Promise<void>;
    drawObject(obj: ObjectType, rect: IRectangle, thickness: number, inverted: boolean): Promise<unknown>;
    setRenderMode(mode: RenderMode): Promise<unknown>;
    requestTransferRAMToDisplay(): Promise<unknown>;
    drawPixels(x: number, y: number, width: number, pixels: Uint8Array): Promise<void>;
}
