"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JLCSlomoElite = exports.FullScreen = exports.Buttons = exports.TBarResolution = exports.RenderMode = exports.ColorMode = exports.ShuttleMode = exports.FontSize = exports.ObjectType = void 0;
const serialport_1 = require("serialport");
const protocol_1 = require("./protocol");
Object.defineProperty(exports, "ColorMode", { enumerable: true, get: function () { return protocol_1.ColorMode; } });
Object.defineProperty(exports, "FontSize", { enumerable: true, get: function () { return protocol_1.FontSize; } });
Object.defineProperty(exports, "ObjectType", { enumerable: true, get: function () { return protocol_1.ObjectType; } });
Object.defineProperty(exports, "RenderMode", { enumerable: true, get: function () { return protocol_1.RenderMode; } });
Object.defineProperty(exports, "ShuttleMode", { enumerable: true, get: function () { return protocol_1.ShuttleMode; } });
Object.defineProperty(exports, "TBarResolution", { enumerable: true, get: function () { return protocol_1.TBarResolution; } });
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
const tcp_client_1 = require("./tcp-client");
const tcp_server_1 = require("./tcp-server");
const Buttons = __importStar(require("./buttons"));
exports.Buttons = Buttons;
exports.FullScreen = { x: 0, y: 0, width: 0xF0, height: 0x7F };
class JLCSlomoElite extends tiny_typed_emitter_1.TypedEmitter {
    constructor(port) {
        super();
        this.protocol = new protocol_1.JLCProtocol();
        this.connected_ = false;
        switch (port.type) {
            case 'serial':
                this.port = this.setupSerialPort(port);
                break;
            case 'tcp-client':
                this.port = this.setupTCPClient(port);
                break;
            case 'tcp-server':
                this.port = this.setupTCPServer(port);
                break;
        }
        this.protocol.on('receive-button', (button, down) => {
            this.emit('button', button, down);
        });
        this.protocol.on('receive-jog', (jog) => {
            this.emit('jog', jog);
        });
        this.protocol.on('receive-shuttle', (shuttle) => {
            this.emit('shuttle', shuttle);
        });
        this.protocol.on('receive-tbar', (tbar) => {
            this.emit('tbar', tbar);
        });
    }
    get connected() {
        return this.connected_;
    }
    setupSerialPort(settings) {
        const port = new serialport_1.SerialPort({
            autoOpen: false,
            baudRate: 230400,
            path: settings.port,
            parity: 'odd',
            stopBits: 1,
            dataBits: 8
        });
        return port;
    }
    setupTCPClient(settings) {
        const port = new tcp_client_1.TCPClient(settings);
        port.on('error', (err) => {
            console.error(err);
        });
        return port;
    }
    setupTCPServer(settings) {
        return new tcp_server_1.TCPServer(settings);
    }
    openPort() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.port.open((err) => {
                    if (err)
                        reject(err);
                    resolve(null);
                });
            });
        });
    }
    dispose() {
        this.port.close();
    }
    open() {
        return __awaiter(this, void 0, void 0, function* () {
            this.port.addListener('connected', () => {
                this.connected_ = true;
                this.emit('connected');
            });
            this.port.addListener('disconnected', () => {
                this.connected_ = false;
                this.emit('disconnected');
            });
            yield this.openPort();
            if (this.port instanceof serialport_1.SerialPort) {
                this.connected_ = true;
            }
            this.port.addListener('data', (data) => {
                this.protocol.parseDataPacket(data);
            });
        });
    }
    clear() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.send(this.protocol.clear());
        });
    }
    send(data) {
        return new Promise((resolve, reject) => {
            this.port.write(data, 'binary', (err) => {
                if (err)
                    reject(err);
                resolve(null);
            });
        });
    }
    requestSystemInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.send(this.protocol.requestSystemInfo());
                const handle = setTimeout(() => {
                    reject(new Error('Timeout'));
                    this.protocol.removeAllListeners('receive-controller-info');
                }, 1000);
                this.protocol.once('receive-controller-info', (info) => {
                    clearTimeout(handle);
                    resolve(info);
                });
            });
        });
    }
    drawText(data_1) {
        return __awaiter(this, arguments, void 0, function* (data, options = {}) {
            yield this.send(this.protocol.writeText(data, options));
        });
    }
    setShuttleMode(mode) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.send(this.protocol.setShuttleMode(mode));
        });
    }
    setButtonColor(button, color) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.send(this.protocol.setButtonColor(button, color));
        });
    }
    setTBarResolution(resolution) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.send(this.protocol.setTBarResolution(resolution));
        });
    }
    drawObject(obj, rect, thickness, inverted) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.send(this.protocol.drawObject(obj, rect, thickness, inverted));
        });
    }
    setRenderMode(mode) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.send(this.protocol.setRenderMode(mode));
        });
    }
    requestTransferRAMToDisplay() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.send(this.protocol.transferRAMToDisplay());
                const handle = setTimeout(() => {
                    reject(new Error('Timeout'));
                    this.protocol.removeAllListeners('receive-transfer-complete');
                }, 1000);
                this.protocol.once('receive-transfer-complete', () => {
                    clearTimeout(handle);
                    resolve(null);
                });
            });
        });
    }
    drawPixels(x, y, width, pixels) {
        return __awaiter(this, void 0, void 0, function* () {
            const cmds = this.protocol.drawPixels(x, y, width, pixels);
            for (let i = 0; i < cmds.length; i++) {
                yield this.send(cmds[i]);
            }
        });
    }
}
exports.JLCSlomoElite = JLCSlomoElite;
//# sourceMappingURL=index.js.map