import { TypedEmitter } from 'tiny-typed-emitter';
export declare const SCREEN_WIDTH = 240;
export declare const SCREEN_HEIGHT = 127;
export interface ControllerInfo {
    rev: number;
    unitType: 'Slomo Elite' | 'Slomo Elite C';
    wheelType: 'Clutch' | 'Non-clutch';
    tbarType: 'High resolution' | 'Analog';
    verticalSwitchType: 'LCD' | 'LED';
    horizontalSwitchType: 'LCD' | 'LED';
    wheelTypeJLCooper: 'Clutch or Stepper' | 'Sony';
    wheelTypeJLCooperMagnetic: 'Clutch' | 'Non-clutch';
}
export interface TextRenderOptions {
    fontSize?: FontSize;
    inverted?: boolean;
    x?: number;
    y?: number;
    spacing?: number;
}
export declare enum FontSize {
    Small = 16,
    Medium = 17,
    Large = 18
}
export declare enum ShuttleMode {
    Disabled = 0,
    Automatic = 1,
    Disengage = 2,
    Engage = 3,
    ShuttleEmulation = 4
}
export declare enum RenderMode {
    Display = 0,
    RAM = 3
}
export declare enum ColorMode {
    Off = 0,
    Red = 1,
    Green = 2,
    Yellow = 3
}
export declare enum TBarResolution {
    Bits6 = 0,
    Bits7 = 1,
    Bits8 = 2,
    Bits9 = 3,
    Bits10 = 4
}
export declare enum ObjectType {
    HorizontalLine = 0,
    VerticalLine = 1,
    FilledRectangle = 2,
    NonFilledRectangle = 3,
    RoundedSwitchSmall = 4,
    RoundedSwitchMedium = 5,
    RoundedSwitchLarge = 6
}
export interface IRectangle {
    x: number;
    y: number;
    width: number;
    height: number;
}
export interface JLCProtocolEvents {
    'receive-controller-info': (info: ControllerInfo) => void;
    'receive-button': (button: number, down: boolean) => void;
    'receive-tbar': (tbar: number) => void;
    'receive-jog': (jog: number) => void;
    'receive-shuttle': (pos: number) => void;
    'receive-transfer-complete': () => void;
}
export declare class JLCProtocol extends TypedEmitter<JLCProtocolEvents> {
    parseDataPacket(data: Uint8Array): void;
    private parseButton;
    private parseJog;
    private parseShuttle;
    private parseTBar;
    private parseReply;
    private parseControllerInfo;
    writeText(data: string, options?: TextRenderOptions): Uint8Array;
    clear(): Uint8Array;
    requestSystemInfo(): Uint8Array;
    requestFirmwareUpdateStart(): Uint8Array;
    setShuttleMode(mode: ShuttleMode): Uint8Array;
    setTBarResolution(resolution: TBarResolution): Uint8Array;
    setButtonColor(button: number, color: ColorMode): Uint8Array;
    drawObject(obj: ObjectType, rect: IRectangle, thickness: number, inverted: boolean): Uint8Array;
    drawPixels(x: number, y: number, width: number, pixels: Uint8Array): Uint8Array[];
    transferRAMToDisplay(): Uint8Array;
    setRenderMode(mode: RenderMode): Uint8Array;
}
