"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JLCProtocol = exports.ObjectType = exports.TBarResolution = exports.ColorMode = exports.RenderMode = exports.ShuttleMode = exports.FontSize = exports.SCREEN_HEIGHT = exports.SCREEN_WIDTH = void 0;
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
exports.SCREEN_WIDTH = 0xF0;
exports.SCREEN_HEIGHT = 0x7F;
var FontSize;
(function (FontSize) {
    FontSize[FontSize["Small"] = 16] = "Small";
    FontSize[FontSize["Medium"] = 17] = "Medium";
    FontSize[FontSize["Large"] = 18] = "Large";
})(FontSize || (exports.FontSize = FontSize = {}));
var ShuttleMode;
(function (ShuttleMode) {
    ShuttleMode[ShuttleMode["Disabled"] = 0] = "Disabled";
    ShuttleMode[ShuttleMode["Automatic"] = 1] = "Automatic";
    ShuttleMode[ShuttleMode["Disengage"] = 2] = "Disengage";
    ShuttleMode[ShuttleMode["Engage"] = 3] = "Engage";
    ShuttleMode[ShuttleMode["ShuttleEmulation"] = 4] = "ShuttleEmulation";
})(ShuttleMode || (exports.ShuttleMode = ShuttleMode = {}));
var RenderMode;
(function (RenderMode) {
    RenderMode[RenderMode["Display"] = 0] = "Display";
    RenderMode[RenderMode["RAM"] = 3] = "RAM";
})(RenderMode || (exports.RenderMode = RenderMode = {}));
var ColorMode;
(function (ColorMode) {
    ColorMode[ColorMode["Off"] = 0] = "Off";
    ColorMode[ColorMode["Red"] = 1] = "Red";
    ColorMode[ColorMode["Green"] = 2] = "Green";
    ColorMode[ColorMode["Yellow"] = 3] = "Yellow";
})(ColorMode || (exports.ColorMode = ColorMode = {}));
var TBarResolution;
(function (TBarResolution) {
    TBarResolution[TBarResolution["Bits6"] = 0] = "Bits6";
    TBarResolution[TBarResolution["Bits7"] = 1] = "Bits7";
    TBarResolution[TBarResolution["Bits8"] = 2] = "Bits8";
    TBarResolution[TBarResolution["Bits9"] = 3] = "Bits9";
    TBarResolution[TBarResolution["Bits10"] = 4] = "Bits10";
})(TBarResolution || (exports.TBarResolution = TBarResolution = {}));
var ObjectType;
(function (ObjectType) {
    ObjectType[ObjectType["HorizontalLine"] = 0] = "HorizontalLine";
    ObjectType[ObjectType["VerticalLine"] = 1] = "VerticalLine";
    ObjectType[ObjectType["FilledRectangle"] = 2] = "FilledRectangle";
    ObjectType[ObjectType["NonFilledRectangle"] = 3] = "NonFilledRectangle";
    ObjectType[ObjectType["RoundedSwitchSmall"] = 4] = "RoundedSwitchSmall";
    ObjectType[ObjectType["RoundedSwitchMedium"] = 5] = "RoundedSwitchMedium";
    ObjectType[ObjectType["RoundedSwitchLarge"] = 6] = "RoundedSwitchLarge";
})(ObjectType || (exports.ObjectType = ObjectType = {}));
class JLCProtocol extends tiny_typed_emitter_1.TypedEmitter {
    parseDataPacket(data) {
        switch (data[0]) {
            case 0x80:
                this.parseButton(data);
                break;
            case 0x81:
                this.parseJog(data);
                break;
            case 0x82:
                this.parseShuttle(data);
                break;
            case 0x83:
                this.parseTBar(data);
                break;
            case 0x84:
                this.emit('receive-transfer-complete');
                break;
            case 0xf0:
                this.parseReply(data);
                break;
        }
    }
    parseButton(data) {
        const down = data[1] === 0x00;
        const button = data[2];
        this.emit('receive-button', button, down);
    }
    parseJog(data) {
        let jog = data[1];
        // jog is 2's complement 7 bit
        if (jog & 0x40) {
            jog = jog - 0x80;
        }
        this.emit('receive-jog', jog);
    }
    parseShuttle(data) {
        let pos = data[1];
        // pos is 2's complement 7 bit
        if (pos & 0x40) {
            pos = pos - 0x80;
        }
        this.emit('receive-shuttle', pos);
    }
    parseTBar(data) {
        let lsb = data[2]; // 7 bits
        let msb = data[1] || 0;
        let tbar = (msb << 7) | lsb;
        this.emit('receive-tbar', tbar);
    }
    parseReply(data) {
        switch (data[3]) {
            case 0x05:
                this.parseControllerInfo(data);
                break;
        }
    }
    parseControllerInfo(data) {
        const revInteger = data[4];
        const revFraction = data[5];
        const rev = revInteger + revFraction / 100;
        const cfg = data[6];
        const unitType = (cfg & 0x01) ? 'Slomo Elite C' : 'Slomo Elite';
        const wheelType = (cfg & 0x02) ? 'Clutch' : 'Non-clutch';
        const tbarType = (cfg & 0x04) ? 'High resolution' : 'Analog';
        const verticalSwitchType = (cfg & 0x08) ? 'LCD' : 'LED';
        const horizontalSwitchType = (cfg & 0x10) ? 'LCD' : 'LED';
        const wheelTypeJLCooper = (cfg & 0x20) ? 'Sony' : 'Clutch or Stepper';
        const wheelTypeJLCooperMagnetic = (cfg & 0x40) ? 'Clutch' : 'Non-clutch';
        const info = {
            rev,
            unitType,
            wheelType,
            tbarType,
            verticalSwitchType,
            horizontalSwitchType,
            wheelTypeJLCooper,
            wheelTypeJLCooperMagnetic
        };
        this.emit('receive-controller-info', info);
    }
    writeText(data, options = {}) {
        const x = (options.x || 0) % 256;
        const y = (options.y || 0) % 256;
        const spacing = (options.spacing || 1) % 5;
        const invertedByte = options.inverted ? 0x01 : 0x00;
        const fontSizeByte = options.fontSize || FontSize.Small;
        const arr = new Uint8Array([0xf0, 0x15, 0x31, 0x01, fontSizeByte, x, y, spacing, invertedByte]);
        const text = Buffer.from(data, 'ascii');
        // convert text to nibbles
        const nibbles = new Uint8Array(text.length * 2);
        for (let i = 0; i < text.length; i++) {
            const byte = text[i];
            nibbles[i * 2] = byte & 0x0f;
            nibbles[i * 2 + 1] = byte >> 4;
        }
        const res = new Uint8Array([...arr, ...nibbles, 0xf7]);
        return res;
    }
    clear() {
        return new Uint8Array([0xf0, 0x15, 0x31, 0x00, 0x00, 0xf7]);
    }
    requestSystemInfo() {
        return new Uint8Array([0xf0, 0x15, 0x31, 0x06, 0xf7]);
        ;
    }
    requestFirmwareUpdateStart() {
        // FEh 55h AAh 01h PCh PCl cps
        // PCh = 0x01
        // PCl = 0x31
        // cps = 2's complement checksum of previous bytes
        const arr = new Uint8Array([0xfe, 0x55, 0xaa, 0x01, 0x31, 0x00]);
        const checksum = arr.reduce((acc, val) => acc + val, 0);
        const cps = (256 - (checksum % 256)) % 256;
        arr.set([cps], 5);
        return arr;
    }
    setShuttleMode(mode) {
        return new Uint8Array([0xf0, 0x15, 0x31, 0x08, mode, 0xf7]);
    }
    setTBarResolution(resolution) {
        const arr = new Uint8Array([0xf0, 0x15, 0x31, 0x0C, resolution, 0xf7]);
        return arr;
    }
    setButtonColor(button, color) {
        //F0h 15h 31h 04h vv ln F7h
        const arr = new Uint8Array([0xf0, 0x15, 0x31, 0x04, color, button, 0xf7]);
        return arr;
    }
    drawObject(obj, rect, thickness, inverted) {
        const sx = rect.x % 256;
        const sy = rect.y % 256;
        const ex = Math.min(rect.x + rect.width, exports.SCREEN_WIDTH) % 256;
        const ey = Math.min(rect.y + rect.height, exports.SCREEN_HEIGHT) % 256;
        const arr = new Uint8Array([0xf0, 0x15, 0x31, 0x02, obj, sx, sy, ex, ey, thickness % 256, inverted ? 0x01 : 0x00, 0xf7]);
        return arr;
    }
    drawPixels(x, y, width, pixels) {
        const sx = x % 256;
        let sy = y % 256;
        const ex = Math.min(x + width, exports.SCREEN_WIDTH);
        const croppedWidth = ex - x;
        // split width into 3 nibbles
        const w = Math.floor(croppedWidth / 8);
        const w3 = (w >> 8) & 0x0f;
        const w2 = (w >> 4) & 0x0f;
        const w1 = w & 0x0f;
        const height = pixels.length / width;
        const res = [];
        for (let i = 0; i < height; i++) {
            const row = pixels.slice(i * width, (i + 1) * width);
            // pack pixels into bits
            const packed = new Uint8Array(croppedWidth / 8);
            for (let i = 0; i < croppedWidth; i++) {
                const byte = row[i];
                for (let j = 0; j < 8; j++) {
                    const bit = (byte >> j) & 0x01;
                    packed[i / 8] |= bit << j;
                }
            }
            const nibbles = new Uint8Array(packed.length * 2);
            for (let i = 0; i < packed.length; i++) {
                const byte = packed[i];
                nibbles[i * 2] = byte & 0x0f;
                nibbles[i * 2 + 1] = byte >> 4;
            }
            const arr = new Uint8Array([0xf0, 0x15, 0x31, 0x03, sx, sy, w1, w2, w3, ...nibbles, 0xf7]);
            sy = (sy + 1) % 256;
            res.push(arr);
        }
        return res;
    }
    transferRAMToDisplay() {
        //F0h 15h 31h 0Bh F7
        return new Uint8Array([0xf0, 0x15, 0x31, 0x0B, 0xf7]);
    }
    setRenderMode(mode) {
        //F0h 15h 31h 0Ah 01h F7h
        return new Uint8Array([0xf0, 0x15, 0x31, 0x0A, mode, 0xf7]);
    }
}
exports.JLCProtocol = JLCProtocol;
//# sourceMappingURL=protocol.js.map