"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TCPClient = void 0;
const net = __importStar(require("net"));
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
class TCPClient extends tiny_typed_emitter_1.TypedEmitter {
    constructor(settings) {
        super();
        this.connected = false;
        this.shouldReconnect = true;
        this.settings = settings;
    }
    open(callback) {
        let callbackCalled = false;
        const callbackOnce = (err) => {
            if (!callbackCalled) {
                callbackCalled = true;
                callback(err);
            }
        };
        this.client = net.createConnection({
            host: this.settings.host,
            port: this.settings.port,
        });
        this.client.on('timeout', () => {
            this.emit('debug', 'timeout');
            this.connected = false;
            this.client.destroy();
            this.connectionClosed();
        });
        this.client.on('close', () => {
            this.emit('debug', 'Socket closed');
            this.connected = false;
            this.connectionClosed();
        });
        this.client.on('connect', () => {
            this.connected = true;
            this.emit('debug', 'Socket connected');
            this.emit('connected');
            callbackOnce(null);
        });
        this.client.on('error', (err) => {
            this.emit('debug', 'Socket Error', err.message);
            this.emit('error', err);
            callbackOnce(err);
        });
        this.client.on('data', (data) => {
            this.emit('data', data);
        });
        this.emit('debug', `Connecting to ${this.settings.host}:${this.settings.port}..`);
    }
    connectionClosed() {
        if (this.shouldReconnect) {
            setTimeout(() => {
                this.emit('debug', 'Reconnecting');
                this.client.connect({ host: this.settings.host, port: this.settings.port });
            }, 1000);
        }
    }
    close() {
        var _a;
        this.shouldReconnect = false;
        (_a = this.client) === null || _a === void 0 ? void 0 : _a.destroy();
    }
    write(data, encoding, callback) {
        if (!this.client) {
            throw new Error('Client not connected');
        }
        if (!this.connected) {
            throw new Error('Client not connected');
        }
        this.client.write(data, encoding, callback);
    }
}
exports.TCPClient = TCPClient;
//# sourceMappingURL=tcp-client.js.map